require 'spec_helper'

module Hikki
  module Adapters
    describe MongoAdapter do
      subject(:adapter) { MongoAdapter.new(connection, db, uuid_generator) }
      let(:connection) { double :connection }
      let(:db) { double :db }
      let(:mongo_collection) { double :mongo_collection, create_index: true, update: true, remove: true }
      let(:uuid_generator) { double :uuid_generator, new: '12345' }
      let(:collection) { 'collection1' }
      before do 
        db.stub(:[]).with(collection).and_return(mongo_collection)
        uuid_generator.stub(:from_string) { |arg| arg }
      end

      describe '#index' do
        it 'returns true' do
          expect(adapter.index(collection, :field1)).to be_true
        end

        it 'adds an index to the collection' do
          adapter.index(collection, :field1)
          expect(mongo_collection).to have_received(:create_index).with('field1')
        end
      end

      describe '#save' do
        context 'when an id is provided in the data' do
          let(:data) { { id: id, field1: 'test', field2: 123 } }
          let(:expected) { { 'id' => id, '_id' => id, 'field1' => 'test', 'field2' => 123 } }
          let(:id) { '1' }

          it 'returns the data' do
            expect(adapter.save(collection, data)).to eq expected
          end

          it 'persists the data in the store' do
            adapter.save(collection, data)
            expect(mongo_collection).to have_received(:update).with({'_id' => '1'}, expected, { upsert: true })
          end
        end

        context 'when an id is not provided in the data' do
          let(:data) { { field1: 'test', field2: 123 } }
          let(:expected) { { 'id' => id, '_id' => id, 'field1' => 'test', 'field2' => 123 } }
          let(:id) { '12345' }

          it 'returns the data with the id added' do
            expect(adapter.save(collection, data)).to eq expected
          end

          it 'persists the data in the store' do
            adapter.save(collection, data)
            expect(mongo_collection).to have_received(:update).with({'_id' => '12345'}, expected, { upsert: true })
          end
        end
      end

      describe '#find' do
        let(:id) { '1' }

        context 'when the id exists' do
          let(:found) { { 'id' => id, '_id' => 1, 'field1' => 'test', 'field2' => 123 } }
          let(:expected) { { 'id' => id, '_id' => '1', 'field1' => 'test', 'field2' => 123 } }
          before { mongo_collection.stub(:find).with({ '_id' => id }).and_return([found]) }

          it 'retrieves the data' do
            expect(adapter.find(collection, id)).to eq expected
          end
        end

        context 'when the id does not exist' do
          let(:expected) { {} }
          before { mongo_collection.stub(:find).with({ '_id' => id }).and_return([]) }

          it 'returns an empty hash' do
            expect(adapter.find(collection, id)).to eq expected
          end
        end
      end

      describe '#all' do
        context 'with a record in the collection' do
          let(:data) { { id: '1', field1: 'test', field2: 123 } }
          let(:found) { { 'id' => '1', '_id' => 1, 'field1' => 'test', 'field2' => 123 } }
          let(:expected) { { 'id' => '1', '_id' => '1', 'field1' => 'test', 'field2' => 123 } }
          let(:cursor) { double :cursor }
          before do
            mongo_collection.stub(:find).and_return(cursor)
            cursor.stub(:sort).with({ '_id' => 1 }).and_return([found])
          end

          it 'returns an array containing the record' do
            expect(adapter.all(collection)).to eq [expected]
          end
        end

        context 'with no records in the collection' do
          let(:cursor) { double :cursor }
          before do
            mongo_collection.stub(:find).and_return(cursor)
            cursor.stub(:sort).with({ '_id' => 1 }).and_return([])
          end

          it 'returns an array containing the record' do
            expect(adapter.all(collection)).to eq []
          end
        end

        context 'with multiple records and paging options specified' do
          let(:cursor) { double :cursor, limit: nil, skip: nil, sort: nil }

          before do
            mongo_collection.stub(:find).and_return(cursor)
            cursor.stub(:sort).and_return(cursor)
          end

          context 'with limit 2' do
            let(:found) { [{'id' => '1', '_id' => 1, 'a' => 1 }, {'id' => '2', '_id' => 2, 'a' => 1 }] }
            let(:expected) { [{'id' => '1', '_id' => '1', 'a' => 1 }, {'id' => '2', '_id' => '2', 'a' => 1 }] }
            before { cursor.stub(:limit).and_return(found) }

            it 'returns the first 2 records' do
              expect(adapter.all(collection, {limit: 2})).to eq expected
            end

            context 'with an offset of 4' do
              let(:found) { [{'id' => '5', '_id' => 5, 'a' => 1 }, {'id' => '6', '_id' => 6, 'a' => 1 }] }
              let(:expected) { [{'id' => '5', '_id' => '5', 'a' => 1 }, {'id' => '6', '_id' => '6', 'a' => 1 }] }
              let(:skip_cursor) { double :skip_cursor, limit: nil }
              before do
                cursor.stub(:skip).with(4).and_return(skip_cursor)
                skip_cursor.stub(:limit).with(2).and_return(found)
              end

              it 'returns the 2 records starting with the offset' do
                expect(adapter.all(collection, {limit: 2, offset: 4})).to eq expected
              end
            end
          end
        end
      end

      describe '#find_by' do
        context 'with a record in the collection' do
          let(:data) { { id: '1', field1: 'test', field2: 123 } }
          let(:found) { { 'id' => '1', '_id' => 1, 'field1' => 'test', 'field2' => 123 } }
          let(:expected) { { 'id' => '1', '_id' => '1', 'field1' => 'test', 'field2' => 123 } }
          let(:cursor) { double :cursor }
          before do
            mongo_collection.stub(:find).with({ 'field1' => 'test' }).and_return(cursor)
            cursor.stub(:sort).with({ '_id' => 1 }).and_return([found])
          end

          it 'returns an array containing the record' do
            expect(adapter.find_by(collection, :field1, 'test')).to eq [expected]
          end
        end

        context 'with no records in the collection' do
          let(:cursor) { double :cursor }
          before do
            mongo_collection.stub(:find).with({ 'field1' => 'test' }).and_return(cursor)
            cursor.stub(:sort).with({ '_id' => 1 }).and_return([])
          end

          it 'returns an array containing the record' do
            expect(adapter.find_by(collection, :field1, 'test')).to eq []
          end
        end

        context 'with multiple records and paging options specified' do
          let(:cursor) { double :cursor, limit: nil, skip: nil, sort: nil }

          before do
            mongo_collection.stub(:find).with({ 'a' => 1 }).and_return(cursor)
            cursor.stub(:sort).and_return(cursor)
          end

          context 'with limit 2' do
            let(:found) { [{'id' => '1', '_id' => 1, 'a' => 1 }, {'id' => '2', '_id' => 2, 'a' => 1 }] }
            let(:expected) { [{'id' => '1', '_id' => '1', 'a' => 1 }, {'id' => '2', '_id' => '2', 'a' => 1 }] }
            before { cursor.stub(:limit).and_return(found) }

            it 'returns the first 2 records' do
              expect(adapter.find_by(collection, :a, 1, {limit: 2})).to eq expected
            end

            context 'with an offset of 4' do
              let(:found) { [{'id' => '5', '_id' => 5, 'a' => 1 }, {'id' => '6', '_id' => 6, 'a' => 1 }] }
              let(:expected) { [{'id' => '5', '_id' => '5', 'a' => 1 }, {'id' => '6', '_id' => '6', 'a' => 1 }] }
              let(:skip_cursor) { double :skip_cursor, limit: nil }
              before do
                cursor.stub(:skip).with(4).and_return(skip_cursor)
                skip_cursor.stub(:limit).with(2).and_return(found)
              end

              it 'returns the 2 records starting with the offset' do
                expect(adapter.find_by(collection, :a, 1, {limit: 2, offset: 4})).to eq expected
              end
            end
          end
        end
      end

      describe '#remove' do
        it 'returns true' do
          expect(adapter.remove(collection, '1')).to be_true
        end

        it 'removes the record from the store' do
          adapter.remove(collection, '1')
          expect(mongo_collection).to have_received(:remove).with({'_id' => '1'})
        end
      end

      describe '#remove_all' do
        it 'returns true' do
          expect(adapter.remove_all(collection)).to be_true
        end

        it 'removes all of the records from the store' do
          adapter.remove_all(collection)
          expect(mongo_collection).to have_received(:remove).with()
        end
      end

    end
  end
end